# -*- coding: UTF-8 -*-

from datetime import datetime

from zope.interface import implements
from zope.i18nmessageid import Message

from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile

from Products.CMFCore.utils import getToolByName

from Products.CMFPlone.i18nl10n import utranslate
from Products.CMFPlone import PloneMessageFactory as _

from Products.Archetypes.Field import decode

from Products.statusmessages import STATUSMESSAGEKEY
from Products.statusmessages.adapter import _decodeCookieValue

from Products.azax import AzaxBaseView as base
from Products.azax import force_unicode
from Products.azax.BeautifulSoup import BeautifulSoup
from Products.azax.plugins.core.interfaces import IKSSCoreCommands

from Products.PloneAzax.commands.interfaces import IIssuePortalMessageCommand

from deprecated import DeprecatedMethod
from interfaces import IPloneAzaxView

class AzaxBaseView(base):
    '''The base view that contains helpers, to be imported
    be other plone products
    '''

    implements(IPloneAzaxView)
    
    header_macros = ZopeTwoPageTemplateFile('browser/macro_wrapper.pt')

    def macroContent(self, macropath, **kw):
        'Renders a macro and returns its text'
        path = macropath.split('/')
        if len(path) < 2 or path[-2] != 'macros':
            raise Exception, 'Path must end with macros/name_of_macro (%s)' % (repr(macropath), )
        # needs string, do not tolerate unicode (causes but at traverse)
        jointpath = '/'.join(path[:-2]).encode('ascii')
        macroobj = self.context.restrictedTraverse(jointpath)
        try:
            the_macro = macroobj.macros[path[-1]]
        except AttributeError, IndexError:
            raise Exception, 'Macro not found'
        #  
        # put parameters on the request, by saving the original context
        self.request.form, orig_form = kw, self.request.form
        content = self.header_macros(the_macro=the_macro)
        self.request.form = orig_form
        # Always encoded as utf-8
        content = force_unicode(content, 'utf')
        return content

    def translateMessage(self, message):
        # XXX David: there must be another way to get the domain
        domain = getattr(message, 'domain', None)
        if not domain:
            domain = 'plone'
        return utranslate(domain, message, context=self.context)

    def issueAllPortalMessages(self):
        if hasattr(self.request.RESPONSE, 'cookies'):
            cookie = self.request.RESPONSE.cookies.get(STATUSMESSAGEKEY)
            if cookie:
                encodedstatusmessages = cookie['value']
                statusmessages = _decodeCookieValue(encodedstatusmessages)
            else:
                statusmessages = []
            for msg in statusmessages:
                IIssuePortalMessageCommand(self).issuePortalMessage(msg.message)
                self.request.RESPONSE.expireCookie(STATUSMESSAGEKEY, path='/')

    # --
    # Command helpers
    # --

    def issuePortalMessage(self, error):
        '''Issue this portal message'''
        if not isinstance(error, ):
            error = _(error)
           
        IIssuePortalMessageCommand(self).issuePortalMessage(error)
