# -*- coding: UTF-8 -*-
from Acquisition import Implicit

from zope.interface import implements

from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile

from Products.CMFCore.utils import getToolByName

from Products.azax.plugins.core.interfaces import IKSSCoreCommands
from Products.azax.BeautifulSoup import BeautifulSoup

from Products.PloneAzax.commands.interfaces import IRefreshPortletCommand
from azaxview import AzaxBaseView
from interfaces import IPloneAzaxView
from deprecated import DeprecatedMethod

class ResumeContentLoading(Exception):
    'Resume content tab loading'

class Acquirer(Implicit):
    # XXX the next should be best to avoid - but I don't know how!
    __allow_access_to_unprotected_subobjects__ = 1
    main_template = ZopeTwoPageTemplateFile('browser/main_template_standalone.pt')

def acquirerFactory(context):
    return context.aq_chain[0].__of__(Acquirer().__of__(context.aq_parent))

def getCurrentContext(context):
    """ Check if context is default page in folder and/or portal
    """
    # check if context is default page
    # XXX This is deprecated in Plone 3.0 (at least)
    # TO DO: We now need to query the @@plone view (if Plone > 3.0)
    portal = getToolByName(context, 'portal_url').getPortalObject()
    if context.isDefaultPageInFolder() and context != portal:
        context = context.aq_inner.aq_parent
    return context


class ContentView(Implicit, AzaxBaseView):

    implements(IPloneAzaxView)
    
    # --
    # Replacing content region
    # --

    # Override main template in this context
    main_template2 = ZopeTwoPageTemplateFile('browser/main_template_standalone.pt')

    #@staticmethod
    def _filter_action(actions, id, found=None):
        if found is not None:
            return found
        for action in actions:
            if action['id'] == id:
                return action
    _filter_action = staticmethod(_filter_action)    # for zope 2.8 / python 2.3

    def replaceContentRegion(self, tabid, url):
        '''Replace content region by tab id

        Usage::
            ul.contentViews li a:click {
	        evt-click-preventdefault: True;
	        action-server: replaceContentRegion;
	        replaceContentRegion-tabid: nodeAttr(id, true);
	        replaceContentRegion-url: nodeAttr(href);
            }

        REMARK:

        Cheat at the moment: we render down the whole page
        but take out the required part only
        This will be optimized more to replace the main template
        for the context of the call

        Warning when we leave the page is not implemented.
        '''
        try:
            # XXX The next checks could be left out - but we won't be able to change the tabs.
            # This could be solved with not using the tabs or doing server side quirks.
            # This affect management screens, for example, that are not real actions.
            if not tabid or tabid == 'content':
                raise ResumeContentLoading
            if not tabid.startswith('contentview-'):
                raise RuntimeError, 'Not a valid contentview id "%s"' % tabid
            #
            if not url.startswith('http://'):
                raise ResumeContentLoading
            path = url.split('/')
            # make the wrapping for the context, to overwrite main_template
            # note we have to use aq_chain[0] *not* aq_base.
            # XXX however just context would be good too? Hmmm
            wrapping = acquirerFactory(self.context)
            ##wrapping = self.context.aq_inner
            # Figure out the template to render.
            if list(path[3:]) ==  list(self.context.getPhysicalPath()[1:]):
                # target is the default view of the method.
                # url is like: ['http:', '', 'localhost:9777', 'kukitportlets', 'prefs_users_overview']
                # physical path is like: ('', 'kukitportlets')
                template = wrapping
            else:
                # see if it is a method on the same context object...
                # url is like: ['http:', '', 'localhost:9777', 'kukitportlets', 'prefs_users_overview']
                # physical path is like: ('', 'kukitportlets')
                if list(path[3:-1]) !=  list(self.context.getPhysicalPath()[1:]):
                    raise ResumeContentLoading
                method = path[-1]
                # Action method may be a method alias: Attempt to resolve to a template.
                try:
                    method = self.context.aq_inner.getTypeInfo().queryMethodID(method, default=method)
                except AttributeError:
                    # Don't raise if we don't have a CMF 1.5 FTI
                    pass
                template = wrapping.restrictedTraverse(method)
            # We render it
            content = template()
            # Now. We take out the required node from it!
            # We need this in any way, as we don't know if the template
            # actually used main_template! In that case we would have
            # the *whole* html which is wrong.
            soup = BeautifulSoup(content)
            replace_id = 'region-content'
            tag = soup.find('div', id=replace_id)
            if tag is None:
                raise RuntimeError, 'Result content did not contain <div id="%s">' % replace_id
            # now we send it back to the client
            result = unicode(tag)
            ##result = unicode(content, 'utf')
            ksscore = IKSSCoreCommands(self)
            ksscore.replaceHTML(ksscore.getHtmlIdSelector(replace_id), result)
            # to remove old tab highlight,...
            ksscore.setAttribute(ksscore.getCssSelector("ul.contentViews li"), name='class', value='plain');
            # ... and put the highlight to the newly selected tab
            ksscore.setAttribute(ksscore.getHtmlIdSelector(tabid), name='class', value='selected');
            return self.render()
        except ResumeContentLoading:
            # empty tabid mean we cannot do a thing this way
            # so we return to the page, that will submit back
            # (tabid == 'content' occurs if we are recursive on parent)
            ##self.addCommand('plone-followLink', self.getSelector('samenode', ''));
            ##return self.render()
            # XXX now, by just re-raising the exception, the client will execute
            # the handler set up from kss.
            raise

    def changeViewTemplate(self, url):
        '''Replace content region after selecting template from drop-down.
        
        Usage::
            dl#templateMenu dd a:click {
            evt-click-preventdefault: True;
            action-server: changeViewTemplate;
            changeViewTemplate-url: nodeAttr(href);
            }

        REMARK:

        Cheat at the moment: we render down the whole page
        but take out the required part only
        This will be optimized more to replace the main template
        for the context of the call

        Warning when we leave the page is not implemented.
        '''
        templateid = url.split('templateId=')[-1].split('&')[0]
        context = getCurrentContext(self.context)
        wrapping = acquirerFactory(context)
        # XXX I believe selectViewTemplate script will be replaced by an
        # adapter or a view in the new implementation of CMFDynamicFTI
        context.selectViewTemplate(templateid)
        # Figure out the template to render.
        template = wrapping.restrictedTraverse(templateid)
        # We render it
        content = template()
        # Now. We take out the required node from it!
        # We need this in any way, as we don't know if the template
        # actually used main_template! In that case we would have
        # the *whole* html which is wrong.

        soup = BeautifulSoup(content)
        replace_id = 'content'
        tag = soup.find('div', id=replace_id)
        if tag is None:
            raise RuntimeError, 'Result content did not contain <div id="%s">' % replace_id
        # now we send it back to the client
        result = unicode(tag)
        ##result = unicode(content, 'utf')
        ksscore = IKSSCoreCommands(self)
        ksscore.replaceHTML(ksscore.getHtmlIdSelector(replace_id), result)

        self.issueAllPortalMessages()
        self.cancelRedirect()
        # XXX We need to take care of the URL history here,
        # Fo instance if we come from the edit page and change the view we
        # stay on the edit URL but with a view page
        return self.render()

    def cutObject(self):
        context = getCurrentContext(self.context)
        context.object_cut()
        self.replaceMenu()
        self.issueAllPortalMessages()
        self.cancelRedirect()
        return self.render()

    def copyObject(self):
        context = getCurrentContext(self.context)
        context.object_copy()
        self.replaceMenu()
        self.issueAllPortalMessages()
        self.cancelRedirect()
        return self.render()

    def changeWorkflowState(self, url):
        action = url.split("workflow_action=")[-1].split('&')[0]
        context = self.context
        context.content_status_modify(action)
        self.replaceMenu()
        IRefreshPortletCommand(self).refreshPortlet('navigation', 'navigation-tree')
        self.issueAllPortalMessages()
        self.cancelRedirect()
        return self.render()

    def replaceMenu(self):
        # render it
        menu_body = self.macroContent('global_contentmenu/macros/contentMenu')
        # Good. Now, unfortunately we don't have any marker on the outside div.
        # So we just select the <dl> for insertion.
        # This could be spared with smarter templating.
        result = unicode(menu_body)
        # Command the replacement
        ksscore = IKSSCoreCommands(self)
        ksscore.replaceInnerHTML(ksscore.getCssSelector('div#content div.contentActions'), result)

