# Copyright (c) 2005-2006
# Authors:
#   Godefroid Chapelle <gotcha@bubblenet.be>
#   Tarek Ziade <tz@nuxeo.com>
#   Balazs Ree <ree@greenfinity.hu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
'''\
AZAX Base view class

All the implementations should create a specialization
of this class when building their browser views.

The policy is that a method should build up a command
set on available methods and then return self.render().

The default command set is implemented in the base class
as well.
'''

try:
    from Products.Five import BrowserView
except ImportError:
    from zope.app.publisher.browser import BrowserView

from zope.interface import implements
from zope.app import zapi

from interfaces import IAzaxView
from commands import AzaxCommands

class AzaxBaseView(BrowserView):
    """ Base azax view

    this allows setting up the content of the response, and then
    generate it out.
    """
    
    implements(IAzaxView)

    def __init__(self, context, request):
        BrowserView.__init__(self, context, request)
        self.commands = AzaxCommands()

    def _set_context(self, context):
        self._context = [context]

    def _get_context(self):
        return self._context[0]

    context = property(_get_context, _set_context)

    def render(self):
        '''All methods must use this to return their command set
        '''
        return self.commands.render(self.request)

    def cancelRedirect(self):
        if self.request.RESPONSE.getStatus() == 302:
            # Try to not redirect if requested
            self.request.RESPONSE.setStatus(200)

    def getCommands(self):
        return self.commands


class AzaxViewAdapter:
    
    implements(IAzaxView)

    def __init__(self, view):
        self.view = view
        self.context = self.view.context
        self.request = self.view.request
        self.commands =  self.view.commands
    
