/*
* Copyright (c) 2005-2006
* Authors:
*   Godefroid Chapelle <gotcha@bubblenet.be>
*   Florian Schulze <florian.schulze@gmx.net>
*   Balázs Reé <ree@greenfinity.hu>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as published
* by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
*/

/* State vars srorage. This can be used from kss via a method. */

kukit.statevars = {};

/* instantiate request manager */

kukit.requestManager = new kukit.rm.RequestManager();

kukit.RuleSheetLink = function(href, res_type) {
    this.href = href;
    this.res_type = res_type;
};

kukit.getRuleSheetLinks = function() {
    var nodes = document.getElementsByTagName("link");
    var results = [];
    for (var i=0; i<nodes.length; i++) {
        if (nodes[i].rel=="kukit" || nodes[i].rel=="k-stylesheet" ) {
            var res_type = null;
            // Resource syntax is decided on type attribute.
            if((nodes[i].type == 'text/css') || (nodes[i].type == 'text/kss')) {
                res_type = 'kss';
            }
            results[results.length] = new kukit.RuleSheetLink(nodes[i].href, res_type);
        }
    }
    return results;
};

kukit.rules = new Array();
kukit.rule_processors = new Array();

kukit.calculateBase = function() {
    try {
        var _dummy = document;
        _dummy = window;
    } catch (e) {
        // testing or what
        return;
    }
    var nodes = document.getElementsByTagName("base");
    if (nodes.length == 0) {
        var base = window.location.href;
        var pieces = base.split('/');
        pieces.pop();
        kukit.base = pieces.join('/');
    } else {
        kukit.base = nodes[0].href;
    }
};

kukit.calculateBase();

// table from res_type to rule processor
// we could make this as a registry but no sense at the moment
kukit.ruleProcessorClasses = {};

kukit.initializeRules = function() {
    if (window.kukitRulesInitialized) {
        // Refuse to initialize a second time.
        kukit.log('Subsequent call to initializeRules is ignored');
        return;
    }
    window.kukitRulesInitialized = true;
    // load the rulesheets
    var rulelinks = kukit.getRuleSheetLinks();
    kukit.log("Nr of Rule Sheet Links: "+rulelinks.length);
    for (var i=0; i<rulelinks.length; i++) {
        var rulelink = rulelinks[i];
        var ruleProcessorClass = kukit.ruleProcessorClasses[rulelink.res_type];
        if (ruleProcessorClass) {
            kukit.log("Start loading and processing " + rulelink.href + " resource type " + rulelink.res_type);
            var ruleprocessor = new ruleProcessorClass(rulelink.href);
            var ts_start = (new Date()).valueOf();
            ruleprocessor.load();
            var ts_loaded = (new Date()).valueOf();
            // Parsing error is also logged.
            try {
                ruleprocessor.parse();
            } catch(e) {
                if (e.name == 'ParsingError') {
                    // Log the message
                    var msg = 'Error parsing ' + rulelink.res_type + ' at ' + rulelink.href + ' : ' + e.toString();
                    kukit.logFatal(msg);
                    // just throw it too...
                    throw msg;
                } else {
                    throw e;
                }
            }
            var ts_end = (new Date()).valueOf();
            kukit.log("Finished loading and processing " + rulelink.href + " resource type " + rulelink.res_type + ' in ' + (ts_loaded - ts_start) + ' + ' + (ts_end - ts_loaded) + ' ms');
            kukit.rule_processors[kukit.rule_processors.length] = ruleprocessor;
        } else {
            kukit.logError("Ignoring rulesheet " + rulelink.href + ' of type ' + rulelink.res_type);
        }
    }
    // Event setup errors are also logged.
    try {
        kukit.setupEvents();
    } catch(e) {
        if (e.name == 'RuleMergeError' || e.name == 'EventBindError') {
            // Log the message
            var msg = 'Error setting up events: ' + e.toString();
            kukit.logFatal(msg);
            // just throw it too...
            throw msg;
        } else {
            throw e;
        }
    }
};

kukit.bindScheduler = new kukit.ut.SerializeScheduler();

kukit.setupEvents = function(in_nodes) {
    var f = function() {
        kukit._setupEvents(in_nodes);
    };
    var targetmsg;
    var found = false;
    if ( ! in_nodes) {
        targetmsg = 'document';
        found = true;
    } else {
        targetmsg = 'node subtrees ';
        for (var i=0; i<in_nodes.length; i++) {
            var node = in_nodes[i];
            if (node.nodeType == 1) {
                if (! found) {
                    found = true;
                } else {
                    targetmsg += ', '; 
                }
                targetmsg += node.tagName.toLowerCase();
            }
        }
    }
    if (found) {
        kukit.bindScheduler.addPre(f, 'setting up events for ' + targetmsg);
    }
};

kukit._setupEvents = function(in_nodes) {
    // Decide phase. 1=initial, 2=insertion.
    var phase;
    if (typeof(in_nodes) == 'undefined') {
        phase = 1;
    } else {
        phase = 2;
    }
    var rules = kukit.rules;
    var ruletable = new kukit.rd.RuleTable();
    for (var y=0; y < rules.length; y++) {
        rules[y].mergeForSelectedNodes(ruletable, phase, in_nodes);
    }
    // bind special selectors first
    if (phase == 1) {
        kukit.rd.methodTable.bindall(phase);
    }
    // finally bind the merged events
    ruletable.bindall(phase)
};

kukit.calculateAbsoluteURL = function(url) {
    if (url.search("http://") == 0) {
        return url;
    } else {
        var result = kukit.base + '/' + url;
        return result;
    }
};


/* Event handling */

/* extracted from Plone */
// cross browser function for registering event handlers
kukit.registerEventListener = function(elem, event, func) {
    if (elem.addEventListener) {
        elem.addEventListener(event, func, false);
        return true;
    } else if (elem.attachEvent) {
        var result = elem.attachEvent("on"+event, func);
        return result;
    }
    // maybe we could implement something with an array
    return false;
};

if (typeof(window) != 'undefined') {
    kukit.registerEventListener(window, "load", kukit.initializeRules);
}

/* Server notification */

// Backparms can be used on command execution.
kukit.notifyServer = function(url, params, oper) {
    var sendHook = function(queueItem) {
        // store the queue reception on the oper
        oper.queueItem = queueItem;
        kukit.reallyNotifyServer(url, params, oper);
    };
    var timeoutHook = function(queueItem) {
        // store the queue reception on the oper
        oper.queueItem = queueItem;
        kukit.processError(oper);
    };
    kukit.requestManager.notifyServer(sendHook, url, timeoutHook);
};

kukit.reallyNotifyServer = function(url, params, oper) {
    // make a deferred callback
    var domDoc = new XMLHttpRequest();
    var notifyServer_done  = function() {
        kukit.notifyServer_done(domDoc, oper);
    };
    // convert params
    var query = new kukit.fo.FormQuery();
    for (var key in params) {
        query.appendElem(key, params[key]);
    }
    var encoded = query.encode();
    // sending form
    var ts = new Date().getTime();
    //kukit.logDebug('TS: '+ts);
    var tsurl = url + "?kukitTimeStamp=" + ts;
    domDoc.open("POST", tsurl, true);
    domDoc.onreadystatechange = notifyServer_done;
    domDoc.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    domDoc.send(encoded);
};

kukit.notifyServer_done = function(domDoc, oper) {
    if (domDoc.readyState == 4)
    {
        // notify the queue that we are done
        var success = oper.queueItem.receivedResult()
        // We only process if the response has not been timed
        // out by the queue in the meantime.
        if (success) {
            // catch the errors otherwise won't get logged.
            // In FF they seem to get swallowed silently.
            try {
                // process the results
                kukit.processResult(domDoc, oper);
            } catch(e) {
                if (e.name == 'RuleMergeError' || e.name == 'EventBindError') {
                    // Log the message
                    var msg = 'Error setting up events: ' + e.toString();
                    kukit.logFatal(msg);
                    // just throw it too...
                    throw msg;
                } else {
                    kukit.logError('Unhandled error during command execution: ' + e);
                    // also IE acts foul on thrown errors
                    // but at least mumbles something
                    throw e;
                }
            }
        }
/*  } else {
        kukit.logDebug('Request arrived with readyState = ' + domDoc.readyState); */
    }
};

kukit.getNsTags = function(domXml, tagName) {
    if (domXml.getElementsByTagNameNS) { 
        tags = domXml.getElementsByTagNameNS('http://www.kukit.org/commands/1.0',
            tagName);
    } else {
        //IE does not know DOM2
        tags = domXml.getElementsByTagName('kukit:' + tagName);
    }
    return tags;
};

kukit.processResult = function(domDoc, oper) {
    var domXml = domDoc.responseXML;
    // for IE, an empty response is produced: we need to check the html tag
    // On Opera, we don't even get to here: we can only catch timeouts elsewhere
    if (domXml == null || typeof(domXml) == 'undefined' || kukit.getNsTags(domXml, 'commands').length != 1) {
        // This will happen in case there are errors in the response
        // Possible cases:
        //   1. No response from server, and browser notifies us with an empty response
        //   2. There was an error on the server, and we got an error message as response
        // Call the error handler.
        kukit.processError(oper);
        return;
    }
    // Opera <= 8.5 does not have the parseError attribute, so check for it first
    if (domXml.parseError && (domXml.parseError != 0)) {
        kukit.logError(Sarissa.getParseErrorText(domXml));
        return;
    }
    commands = kukit.getNsTags(domXml, 'command');
    if (commands.length == 0) {
        kukit.logWarning('No commands in kukit response');
        return;
    }
    var command_processor = new kukit.CommandProcessor();
    command_processor.parseCommands(commands, domDoc);
    command_processor.executeCommands(oper);
};

kukit.processError = function(oper) {
    var error_action = null;
    if (oper.eventrule) {
        var error_action = oper.eventrule.actions.getErrorActionFor(oper.action);
        }
    if (error_action) {
        kukit.logWarning('Request failed at url ' + oper.queueItem.url + ', rid=' + oper.queueItem.rid + ', will be handled by action "' + error_action.name + '"');
        // Individual error handler was defined. Execute it!
        error_action.execute(oper);
    } else {
        // Unhandled: just log it...
        kukit.logError('Request failed at url ' + oper.queueItem.url + ', rid=' + oper.queueItem.rid);
    }
};

kukit.CommandExecutionError = kukit.ut.exceptionFactory('CommandExecutionError');
kukit.CommandExecutionError.prototype.__superinit__ = kukit.CommandExecutionError.prototype.__init__; 
kukit.CommandExecutionError.prototype.__init__ = function(name, e, command) {
    var kw = this.__superinit__(name, '');
    kw.message = 'Command "' + command.name + '": ' + e.toString();
    return kw
};

kukit.CommandProcessor = function() {
      this.commands = new Array();
};

kukit.CommandProcessor.prototype.parseCommands = function(commands, transport) {
    kukit.log('Parse commands');
    kukit.logDebug('Number of commands: ' + commands.length);
    for (var y=0;y < commands.length;y++) {
        var command = commands[y];
        this.parseCommand(command, transport);
    }
};

kukit.CommandProcessor.prototype.parseCommand = function(command, transport) {
    var selector = "";
    var params = {};
    var name = "";

    selector = command.getAttribute("selector");
    name = command.getAttribute("name");
    type = command.getAttribute("selectorType");
    if (name == null)
        name = "";
    var childNodes = command.childNodes;
    for (var n=0;n < childNodes.length;n++) {
        var childNode = childNodes[n];
        if (childNode.nodeType != 1) 
            continue;
        if (childNode.localName) {
            if (childNode.localName.toLowerCase() != "param") {
                throw 'Bad payload, expected param';
            }
        } else {
            //IE does not know DOM2
            if (childNode.nodeName.toLowerCase() != "kukit:param") {
                throw 'Bad payload, expected kukit:param';
            }
        }        
        data = childNode.getAttribute('name');
        if (data != null) { 
            // Decide if we have a string or a dom parameter
            var childCount = childNode.childNodes.length;
            var result;
            if (childCount == 0) {
                // We take this a string (although this could be dom)
                result = '';
            } else if (childCount == 1 && childNode.firstChild.nodeType == 3) {
                // we have a single text node
                result = childNode.firstChild.nodeValue;
            } else {
                // dom
                result = childNode;
            }
            params[data] = result;
        } else {
            throw 'Bad payload, expected attribute "name"';
        }
    }
    var command = new kukit.cr.makeCommand(selector, name, type, params, transport);
    this.addCommand(command);
}; 

kukit.CommandProcessor.prototype.addCommand = function(command) {
    this.commands[this.commands.length] = command;
};

kukit.CommandProcessor.prototype.executeCommands = function(oper) {
    // node, eventrule, binderinstance are given on oper, in case
    // the command was called up from an event
    if (typeof(oper) == 'undefined' || oper == null) {
        oper = new kukit.op.Oper();
    }
    var commands = this.commands;
    for (var y=0;y < commands.length;y++) {
        var command = commands[y];
        try {
            command.execute(oper); 
        } catch (e) {
            if (e.name == 'RuleMergeError' || e.name == 'EventBindError') {
                throw(e);
            } else {
                // augment the error message
                throw new kukit.CommandExecutionError(e, command); 
            }
        }
    }
};

