/*
* Copyright (c) 2005-2006
* Authors:
*   Godefroid Chapelle <gotcha@bubblenet.be>
*   Florian Schulze <florian.schulze@gmx.net>
*   Balázs Reé <ree@greenfinity.hu>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as published
* by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
* 02111-1307, USA.
*/

/* Supplemental data that the parser builds up */

kukit.rd = {};

kukit.rd.RuleMergeError = kukit.ut.exceptionFactory('RuleMergeError');

kukit.rd.KssSelectorError = kukit.ut.exceptionFactory('KssSelectorError');

kukit.rd.EventBindError = kukit.ut.exceptionFactory('EventBindError');
kukit.rd.EventBindError.prototype.__superinit__ = kukit.rd.EventBindError.prototype.__init__; 
kukit.rd.EventBindError.prototype.__init__ = function(name, message, eventname, eventnamespace) {
    var kw = this.__superinit__(name, message);
    kw.eventname = eventname;
    kw.eventnamespace = eventnamespace;
    kw.message = kw.message + ' when binding event name "' + eventname + '" on namespace "' + eventnamespace + '"';
    return kw
};

kukit.rd.makeId = function(namespace, name) {
    if (namespace == null) {
        namespace = '';
    }
    return '@' + namespace + '@' + name;
};

kukit.rd.makeMergeId = function(id, namespace, name) {
    if (namespace == null) {
        namespace = '';
    }
    return id + '@' + namespace + '@' + name;
};

kukit.rd.KssSelector = function(isEvent, css, name, namespace, id) {
    this.isEventSelector = isEvent;
    this.isMethodSelector = ! isEvent;
    if (! name) {
        throw 'KssSelector must have name';
    }
    if (name.indexOf('@') != -1)
        throw new kukit.rd.KssSelectorError('Kss selector name must not contain @: "' + name + '"');
    if (id && id.indexOf('@') != -1)
        throw new kukit.rd.KssSelectorError('Kss selector id must not contain @: "' + id + '"');
    if (namespace && namespace.indexOf('@') != -1)
        throw new kukit.rd.KssSelectorError('Kss selector namespace must not contain @: "' + namespace + '"');
    if (! isEvent) {
        // method rule
        if (css != 'document' && css != 'behaviour') {
            throw new kukit.rd.KssSelectorError('KssSpecialSelector "' + name + '" must have one of the allowed names');
        }
    }
    this.css = css;
    this.name = name;
    this.namespace = namespace;
    this.classname = null;
    this.id = id;
};

kukit.rd.KssSelector.prototype.setIdAndClass = function() {
    // Sets up id and class on the selector, based on registration info
    this.classname = kukit.er.eventRegistry.get(this.namespace, this.name).classname
    if (this.id == null) {
        // singleton for class
        this.id = kukit.rd.makeId(this.namespace, this.classname);
    }
    // Also set the merge id. The rules with the same merge
    // id should be merged on the same node.
    this.mergeid = kukit.rd.makeMergeId(this.id, this.namespace, this.name);
};

/*
* Kss parameter values. There are two kinds: text and method.
*
* They are evaluated in two phases: check is invoked at parsing,
* allowing the early detection of errors. Evaluate is called
* when the action is to be called. This allows a kss method
* to add any parameter to the action.
*/

kukit.rd.KssTextValue = function(txt) {
    // A text parameter in the format 
    //      key: value;
    this.txt = txt;
};
kukit.rd.KssTextValue.prototype.isMethod = false;
kukit.rd.KssTextValue.prototype.check = function() {
};
kukit.rd.KssTextValue.prototype.evaluate = function(parms, key) {
    parms[key] = this.txt;
};

kukit.rd.KssMethodValue = function(methodname, args) {
    // A method parameter in the format 
    //      key: methodname(v1, v2, ... vn);
    this.methodname = methodname;
    this.args = args;
};
kukit.rd.KssMethodValue.prototype.isMethod = true;
kukit.rd.KssMethodValue.prototype.check = function() {
    // Check syntax
    switch (this.methodname) {
        case 'formVar': {
            if (this.args.length != 2) {
                throw 'formVar method needs 2 arguments (formname, varname)';
            }
        } break;
        case 'currentFormVar': {
            if (this.args.length != 0 && this.args.length != 1) {
                throw 'currentFormVar method needs 0 or 1 argument (varname)';
            }
        } break;
        case 'currentFormVarFromKssAttr': {
            if (this.args.length != 1 && this.args.length != 2) {
                throw 'currentFormVarFromKssAttr method needs 1 or 2 argument (attrname, [recurseParents])';
            }
        } break;
        case 'form': {
            if (this.args.length != 1) {
                throw 'form method needs 1 arguments (formname)';
            }
        } break;
        case 'currentForm': {
            if (this.args.length != 0) {
                throw 'currentFormVar method needs no argument';
            }
        } break;
        case 'nodeAttr': {
            if (this.args.length != 1 && this.args.length != 2) {
                throw 'nodeAttr method needs 1 or 2 argument (attrname, [recurseParents])';
            }
            if (this.args[0].toLowerCase() == 'style') {
                throw 'nodeAttr method does not accept "style" as attrname';
            }
            if (this.args[0].match(/[ ]/)) {
                throw 'attrname parameter in nodeAttr method cannot contain space.';
            }
        } break;
        case 'kssAttr': {
            if (this.args.length != 1 && this.args.length != 2) {
                throw 'nodeAttr method needs 1 or 2 argument (attrname, [recurseParents])';
            }
            if (this.args[0].match(/[ -]/)) {
                throw 'attrname parameter in kssAttr method cannot contain "-" or space.';
            }
        } break;
        case 'nodeContent': {
            if (this.args.length != 0 && this.args.length != 1) {
                throw 'nodeContent method needs 0 or 1 argument (recursive)';
            }
        } break;
        case 'stateVar': {
            if (this.args.length != 1) {
                throw 'stateVar method needs 1 argument (varname)';
            }
        } break;
        case 'pass': {
            if (this.args.length != 1) {
                throw 'pass method needs 1 argument (attrname)';
            }
        } break;
        default: {
            throw 'Not implemented kss parameter method "' + this.methodname + '"';
        } break;
    }
};
kukit.rd.KssMethodValue.prototype.evaluate = function(parms, key, node, defaultparms) {
    // Evaluate into parms.
    switch (this.methodname) {
        case 'formVar': {
            parms[key] = kukit.fo.getFormVarFromNamedForm(this.args[0], this.args[1]);
        } break;
        case 'currentFormVar': {
            if (this.args.length == 1) {
                parms[key] = kukit.fo.getFormVarFromCurrentForm(node, this.args[0]);
            } else {
                // no form var name, just get the value of the node.
                parms[key] = kukit.fo.getValueOfFormElement(node);
            }
        } break;
        case 'currentFormVarFromKssAttr': {
            var argname =  this.args[0];
            var recurseParents = false;
            if (this.args.length == 2) {
                kukit.ut.evalBool(this.args[1], '2nd attribute of currentFormVarForKssAttr must be a boolean');
                recurseParents = this.args[1];
            }
            var formvarname = kukit.dom.getRecursiveAttribute(node, argname, recurseParents, kukit.dom.getKssAttribute);
            parms[key] = kukit.fo.getFormVarFromCurrentForm(node, formvarname);
        } break;
        case 'form': {
            parms[key] = kukit.fo.getAllFormVarsFromNamedForm(this.args[0]);
        } break;
        case 'currentForm': {
            parms[key] = kukit.fo.getAllFormVarsFromCurrentForm(node);
        } break;
        case 'nodeAttr': {
            var argname =  this.args[0];
            var recurseParents = false;
            if (this.args.length == 2) {
                kukit.ut.evalBool(this.args[1], '2nd attribute of nodeAttr must be a boolean');
                recurseParents = this.args[1];
            }
            var value = kukit.dom.getRecursiveAttribute(node, argname, recurseParents, kukit.dom.getAttribute);
            parms[key] = value;
        } break;
        case 'kssAttr': {
            var argname =  this.args[0];
            var recurseParents = false;
            if (this.args.length == 2) {
                kukit.ut.evalBool(this.args[1], '2nd attribute of kssAttr must be a boolean');
                recurseParents = this.args[1];
            }
            var value = kukit.dom.getRecursiveAttribute(node, argname, recurseParents, kukit.dom.getKssAttribute);
            parms[key] = value;
        } break;
        case 'nodeContent': {
            var recursive = false;
            if (this.args.length == 1) {
                recursive = this.args[0];
            }
            parms[key] = kukit.dom.textContent(node, recursive);
        } break;
        case 'stateVar': {
            var key2 = this.args[0];
            var value = kukit.statevars[key2];
            if (typeof(value) == 'undefined') {
                // notfound arguments will get null
                throw 'Nonexistent statevar "'+ key2 +'"';
            }
            parms[key] = value;
        } break;
        case 'pass': {
            var key2 = this.args[0];
            var value = defaultparms[key2];
            if (typeof(value) == 'undefined') {
                // notfound arguments will get null
                throw 'Nonexistent default parm "'+ key2 +'"';
            }
            parms[key] = value;
        } break;
        default: {
            throw 'Ignored kss parameter method.';
        } break;
    }
};

kukit.rd.KssPseudoValue = function(methodname, args) {
    // A method parameter in the format 
    //      methodname(v1)
    this.methodname = methodname;
    this.args = args;
};
kukit.rd.KssPseudoValue.prototype.isMethod = true;
kukit.rd.KssPseudoValue.prototype.check = function() {
};

kukit.rd.EventRuleNr = 0;            // just a counter

kukit.rd.EventRule = function(kss_selector, parms, actions) {
    if (typeof(parms) == 'undefined') {
        // called for merging clone
        this.kss_selector = kss_selector;
    } else {
        this.nr = kukit.rd.EventRuleNr;
        this.mergednr = null;
        kukit.rd.EventRuleNr = this.nr + 1;
        var namestr;
        if (kss_selector.namespace) {
            namestr = kss_selector.namespace + '-' + kss_selector.name;
        } else {
            namestr = kss_selector.name;
        }
        kukit.logDebug("EventRule #" + this.getNr() + ": " + kss_selector.css + ' EVENT=' + namestr);
        this.kss_selector = kss_selector;
        this.parms = parms;
        this.actions = actions;
    }
};

kukit.rd.EventRule.prototype.getNr = function() {
    if (this.mergednr) {
        return this.mergednr;
    } else {
        return this.nr;
    }
};

kukit.rd.EventRule.prototype.mergeForSelectedNodes = function(ruletable, phase, in_nodes) {
    // Select all nodes within the in_nodes for phase==2. (or undefined on initial node, phase==1)
    // Merge itself to the selected nodes.
    if (this.kss_selector.isEventSelector) {
        var nodes = kukit.dom.cssQuery(this.kss_selector.css, in_nodes);
        var counter = 0;
        for (var y=0; y < nodes.length; y++)
        {
            var node = nodes[y];
            // XXX never rebind to any node again!
            // this compensates that cssQuery is returning results out of the subtree
            if (typeof(node._kukitmark) == 'undefined') {
                ruletable.add(node, this);
                counter += 1;
                }
        }
        if (counter > 0) {
            kukit.logDebug('EventRule #' + this.getNr() + ' mergeid ' + this.kss_selector.mergeid + ' selected ' + counter + ' nodes');
        }
    } else if (typeof(in_nodes) == 'undefined') {
        // Method selector. They only need to be handled on the initial
        // pageload, when the in_nodes parameter is ommitted.
        kukit.rd.methodTable.add(this);
    }
};

kukit.rd.EventRule.prototype.getBinderInstance = function() {
    // Gets the event instance for the rule.
    return kukit.er.binderInstanceRegistry.getOrCreateBinder(this.kss_selector.id, this.kss_selector.classname, 
        this.kss_selector.namespace);
};

/*
* bind(node) : calls binder hook on event instance.
*  These hooks are tried in order, if succeeds it must return true:
*
* __bind__(name, parms, func_to_bind, node, eventrule)
* __bind_<name>__(parms, func_to_bind, node, eventrule)
*
* If none succeeds is an error.
*
*/

kukit.rd.EventRule.prototype.bind = function(node) {
    var oper = new kukit.op.Oper()
    oper.node = node;
    oper.binderinstance = this.getBinderInstance();
    oper.eventrule = this;
    oper.parms = {};
    var namespace = this.kss_selector.namespace;
    var name = this.kss_selector.name;
    var methodname = kukit.er.eventRegistry.get(namespace, name).bindmethodname
    if (! methodname) {
        throw new kukit.rd.EventBindError('Method is not defined as bindable', name, namespace);
    }
    var method = oper.binderinstance[methodname];
    if (! method) {
        throw new kukit.rd.EventBindError('Method "' + methodname + '" does not exist', name, namespace);
    }
    var func_to_bind = function(dict) {
        var newoper = oper.clone(dict, true);
        //oper.logDebug();
        newoper.binderinstance._EventBinder_triggerevent(name, newoper);
    };
    // do the binding    
    var bindoper = new kukit.op.Oper()
    bindoper.node = node;
    bindoper.binderinstance = oper.binderinstance;
    bindoper.eventrule = this;
    bindoper.parms = this.parms;
    // Protect the binding for better logging
    try {
        method.call(bindoper.binderinstance, name, func_to_bind, bindoper);
    } catch(e) {
        throw new kukit.rd.EventBindError('Error during binding, reason: [' + e + ']', name, namespace);
    }
    // mark on the instance as bound
    kukit.er.binderInstanceRegistry.markBinding(oper);
};


//
// Merging event rules
//

kukit.rd.EventRule.prototype.isMerged = function() {
    return (this.mergednr != null);
};

kukit.rd.EventRule.prototype.cloneForMerge = function() {
    // Do not touch ourselves, make a new copy for the merge.
    var merged = new kukit.rd.EventRule(this.kss_selector);
    merged.actions = new kukit.rd.ActionSet();
    merged.parms = {};
    merged.mergednr = 'X';
    merged.merge(this);
    merged.mergednr = this.getNr();
    return merged;
};

kukit.rd.EventRule.prototype.merge = function(other) {
    if (! this.isMerged()) {
        throw "Cannot merge into a genuine event rule";
    }
    if (this.kss_selector.isEventSelector) {
        if (this.kss_selector.id != other.kss_selector.id) {
            throw "Differing kss selector ids in event rule merge";
        }
        if (this.kss_selector.classname != other.kss_selector.classname) {
            throw "Differing kss selector classes in event rule merge";
        }
    }
    if (this.kss_selector.name != other.kss_selector.name) {
        throw "Differing kss selector names in event rule merge";
    }
    this.mergednr = this.mergednr + ',' + other.getNr();
    for (var key in other.parms) {
        this.parms[key] = other.parms[key];
    }
    this.actions.merge(other.actions);
    if (this.mergednr[0] != 'X')
        // ignore initial clone-merge
        kukit.logDebug('Merged rule ' + this.mergednr + ' mergeid ' + this.kss_selector.mergeid);
};

kukit.rd.EventRule.prototype.mergeIntoDict = function(dict, key, eventrule) {
    // Merge into the given dictionary by given key.
    // If possible, store the genuine rule first - if not,
    // clone it and do a merge. Never destroy the genuine
    // rules, clone first. This is for efficiency.
    var mergedrule = dict[key];
    if (typeof(mergedrule) == 'undefined') {
        // there was no rule
        dict[key] = eventrule;
    } else {
        // we have to merge the rule
        if (! mergedrule.isMerged()) {
            // Make sure genuine instances are replaced
            mergedrule = mergedrule.cloneForMerge();
            dict[key] = mergedrule;
        }
        mergedrule.merge(eventrule);
    }
};

kukit.rd.ActionSet = function() {
    this.content = {};
};

kukit.rd.ActionSet.prototype.merge = function(other) {
    for (var key in other.content) {
        var action = this.content[key];
        var action2 = other.content[key];
        if (typeof(action) == 'undefined') {
            if (action2.type != 'X') {
                // new action
                action = new kukit.rd.Action();
                this.content[key] = action;
            } else {
                throw new kukit.rd.RuleMergeError('Cannot action-delete unexisting action, "' + key + '"');
            }
        }
        if (action2.type != 'X') {
            // merge the action
            action.merge(action2);
        } else {
            // Delete the action
            this.deleteAction(key);
        }
    }
};

kukit.rd.ActionSet.prototype.execute = function(oper) {
    for (var key in this.content) {
        var action = this.content[key];
        // do not execute error actions!
        if (action.type != 'E') {
            action.execute(oper);
        }
    }
    // Execute the default action in case of there is one but there were no
    // parameters so it was actually not entered as an action object
    // otherwise, it would have been executed from action.execute already
    if (typeof(this.content['default']) == 'undefined') {
        var namespace = oper.eventrule.kss_selector.namespace;
        var name = oper.eventrule.kss_selector.name;
        oper.binderinstance._EventBinder_calldefault(namespace, name, oper);
    }
};

kukit.rd.ActionSet.prototype.getOrCreateAction = function(name) {
    var action = this.content[name];
    if (typeof(action) == 'undefined') {
        action = new kukit.rd.Action();
        action.setName(name);
        this.content[name] = action;
    }
    return action;
};

kukit.rd.ActionSet.prototype.getActionOrNull = function(name) {
    var action = this.content[name];
    if (typeof(action) == 'undefined') {
        action = null;
    }
    return action;
};

kukit.rd.ActionSet.prototype.deleteAction = function(name) {
    var action = this.content[name];
    if (typeof(action) == 'undefined') {
        throw('Action "' + name + '" does not exist and cannot be deleted.');
    }
    delete this.content[name];

};

kukit.rd.ActionSet.prototype.getDefaultAction = function() {
    return this.getActionOrNull('default');
};

kukit.rd.ActionSet.prototype.getErrorActionFor = function(action) {
    // Get the error action of a given action: or null,
    // if the action does not define an error handler.
    return this.getActionOrNull(action.error);
};

kukit.rd.Action = function() {
    this.name = null;
    this.error = null;
    this.parms = {};
    this.type = null;
}

kukit.rd.Action.prototype.setName = function(name) {
    if (this.name != null && this.name != name) {
        throw new kukit.rd.RuleMergeError('Error overriding action name "' + this.name + '" to "' + name + '" (Unmatching action names at merge?)');
    }
    this.name = name;
    if (name == 'default') {
        if (this.type != null && this.type != 'D') {
            throw new kukit.rd.RuleMergeError('Error setting action to default on action "' + this.name + '", current type "' + this.type + '"');
        }
        this.setType('D');
    }
};

kukit.rd.Action.prototype.setType = function(type) {
    // Possible types:
    //  S = server
    //  C = client
    //  E = error / client
    //  D = default (unsettable)
    //  X = cancel action
    if ((type != 'S' && type != 'C' && type != 'E' && type != 'X') ||
            (this.type != null && this.type != type)) {
        throw new kukit.rd.RuleMergeError('Error setting action type on action "' + this.name + '" from "' + this.type + '" to "' + type + '" (Attempt to merge client, server or error actions?)');
    }
    if (this.error != null && this.type != 'S') {
        throw new kukit.rd.RuleMergeError('Error setting action error handler on action "' + this.name + '", this is only allowed on server actions.');
    }
    this.type = type;  
};

kukit.rd.Action.prototype.setError = function(error) {
    if (this.type != null && this.type != 'S') {
        throw new kukit.rd.RuleMergeError('Error setting action error handler on action "' + this.name + '", this is only allowed on server actions.');
    }
    this.error = error;  
};

kukit.rd.Action.prototype.merge = function(other) {
    // Merge to the instance.
    if (other.name != null) { 
        this.setName(other.name);
    }
    if (other.type != null) { 
        this.setType(other.type);
    }
    if (other.error != null) { 
        this.setError(other.error);
    }
    // These are simply overwritten.
    for (var key in other.parms) {
        this.parms[key] = other.parms[key];
    }
};

kukit.rd.Action.prototype.getParms = function(node, defaultparms) {
    // Return the completed action parameters, based on the node
    // if defaultparms is given, its values can be used with the 
    // pass() function.
    var parms = {};
    if (typeof(defaultparms) == 'undefined') {
        defaultparms = {};
    }
    for (var key in this.parms) {
        var kssvalue = this.parms[key]; 
        // evaluate the method parameters into parms
        kssvalue.evaluate(parms, key, node, defaultparms); 
    }
    return parms;
};

kukit.rd.Action.prototype.execute = function(oper) {
    oper = oper.clone({
            'parms': this.getParms(oper.node, oper.parms),
            'action': this
        });
    switch (this.type) {
        case 'D': {
            // Default action.
            var namespace = oper.eventrule.kss_selector.namespace;
            var name = oper.eventrule.kss_selector.name;
            oper.binderinstance._EventBinder_calldefault(namespace, name, oper);
        } break;
        case 'S': {
            // Server action.
            oper.executeServerAction(this.name);
        } break;
        case 'C': {
            // Client action.
            oper.executeClientAction(this.name);
        } break;
        case 'E': {
            // Error action (= client action)
            oper.executeClientAction(this.name);
        } break;
    }
};


/*     Rule table

    Used for binding rules to nodes, and handling the merges.
    It is a two level dictionary.

    There are more rules that match a given node and event id. 
    They will be merged appropriately. The event id is also
    important. The event class must be the same with merge
    rules (within the id).

    To summarize the procedure, each eventrule is added with
    all the nodes that are selected by it. Nothing is executed,
    only merges are done at this time. Finally, all binds are
    done in the second path.

    Event with the same merge id are merged. The merge id is
    a concatenation of the event id and the event name.
*/

// XXX TODO this has to be refactored, since it's all global now

kukit.rd.LoadScheduler = function() {
    this.items = [];
};

kukit.rd.LoadScheduler.prototype.empty = function() {
    return (this.size() == 0);
};

kukit.rd.LoadScheduler.prototype.size = function() {
    return this.items.length;
};

kukit.rd.LoadScheduler.prototype.push = function(f) {
    if (this.items.length >= 100) {
        throw ('Infinite recursion, stack full');
    }
    this.items.push(f);
};

kukit.rd.LoadScheduler.prototype.execute = function() {
    var f = this.items.shift();
    if (f) {
        f();
        return true;
    } else {
        return false;
    }
};

kukit.rd.LoadScheduler.prototype.executeAll = function() {
    var i = 0;
    while(true) {
        var success = this.execute();
        if (! success) {
            break;
        }
        i++;
    }
    return i;
};

kukit.rd.loadScheduler = new kukit.rd.LoadScheduler();

kukit.rd.RuleTable = function() {
    this.nodes = {};
};

kukit.rd.RuleTable.prototype.add = function(node, eventrule) {
    // look up node
    var nodehash = kukit.rd.hashnode(node);
    var nodeval = this.nodes[nodehash];
    if (typeof(nodeval) == 'undefined') {
        nodeval = {'node': node, 'val': {}};
        this.nodes[nodehash] = nodeval;
    }
    // Merge into the dict
    eventrule.mergeIntoDict(nodeval.val, eventrule.kss_selector.mergeid, eventrule);
};

kukit.rd.RuleTable.prototype.bindall = function(phase) {
    // Bind all nodes
    var counter = 0;
    for (var nodehash in this.nodes) {
        var nodeval = this.nodes[nodehash];
        // XXX Mark the node, disabling rebinding in a second round
        nodeval.node._kukitmark = phase;
        for (var id in nodeval.val) {
            var eventrule = nodeval.val[id];
            eventrule.bind(nodeval.node);            
        }
        counter += 1;
    }
    kukit.logDebug('Binding to ' + counter + ' nodes in grand total');
    // Execute the load actions in a deferred manner
    var loadactions = kukit.rd.loadScheduler;
    if (! loadactions.empty()) {
        kukit.logDebug('Start executing delayed load actions');
        var nr = loadactions.executeAll();
        kukit.logDebug('Executed ' + nr + ' load actions');
    }
};

kukit.rd.uid = 0;

kukit.rd.hashnode = function(node) {
    // It is, generally, not possible to use a node as a key.
    // However we try to set this right.
    // We generate an uniqueID on the node. This does not work
    // on MSIE but it already has an uniqueID.
    var id = node.uniqueID;
    if (typeof(id) == 'undefined') {
        id = kukit.rd.uid;
        node.uniqueID = id;
        kukit.rd.uid ++;
    }
    return id;
};

/*
* The method table
* This books the method rules.
*
* Unlike the rule table that is specific for each binding, this is
* unique to the page.
*/

kukit.rd.MethodTable = function() {
    this.content = {};
    this.content['document'] = {};
    this.content['behaviour'] = {};
};

kukit.rd.MethodTable.prototype.add = function(eventrule) {
    // Get the entry by the type which is now at css
    var category = eventrule.kss_selector.css;
    var dict = this.content[category];
    if (typeof(dict) == 'undefined') {
        throw 'Unknown method rule category "' + category + '"';
    }
    // Merge into the corresponding category
    eventrule.mergeIntoDict(dict, eventrule.kss_selector.mergeid, eventrule);
};

kukit.rd.MethodTable.prototype.getMergedRule = function(category, name, binderinstance) {
    // Returns the rule for a given event instance, 
    // Get the entry by category (= document or behaviour)
    var dict = this.content[category];
    if (typeof(dict) == 'undefined') {
        throw 'Unknown method rule category "' + category + '"';
    }
    // look up the rule
    var namespace = binderinstance.__event_namespace__;
    var id = binderinstance.__binder_id__;
    var mergeid = kukit.rd.makeMergeId(id, namespace, name);
    var mergedrule = dict[mergeid];
    if (typeof(mergedrule) == 'undefined') {
        // no error, just return null.
        mergedrule = null;
    }
    return mergedrule;
};

kukit.rd.MethodTable.prototype.bindall = function() {
    // bind document events
    var documentrules = this.content['document'];
    var counter = 0;
    for (var mergeid in documentrules) {
        // bind to null as a node
        documentrules[mergeid].bind(null);
        counter += 1;
    }
    kukit.logDebug('Binding ' + counter + ' special rules in grand total');
};
 
kukit.rd.methodTable = new kukit.rd.MethodTable();

