import zope.component
from zope.component.exceptions import ComponentLookupError
from interfaces import IAzaxPluginRegistry
from zope.interface import implements
# concatresource is an embedded product 
import _concatresource
from concatresource.interfaces import IConcatResourceAddon

def getRegistry(context=None):
    'Convenience method to get or create the registry.'
    try:
        registry = zope.component.getUtility(IAzaxPluginRegistry)
    except ComponentLookupError:
        # Register the global utility, also register it for concatresources
        registry = AzaxPluginRegistry()
        utilities = zope.component.getService(zope.component.servicenames.Utilities)
        utilities.provideUtility(IAzaxPluginRegistry, registry)
        utilities.provideUtility(IConcatResourceAddon, registry, name='kukit.js')
    return registry
    
class AzaxPluginError(Exception):
    pass
    
class JsFile(object):

    def __init__(self, name):
        self.name = name
        self.content = []

class AzaxPluginRegistry(object):
    implements(IAzaxPluginRegistry, IConcatResourceAddon)
    
    def __init__(self):
        self.jsfiles = {}
        self.plugins = {}
    
    def register(self, plugin):
        'Register a plugin'
        # register the plugin
        if self.isRegistered(plugin.plugintype, plugin.name): 
            raise AzaxPluginError, 'Duplicate registration attempt for plugin "%s" of type "%s"' % (plugin.name, plugin.plugintype)
        plugins_for_class = self.plugins[plugin.plugintype]
        plugins_for_class[plugin.name] = plugin
        # register jsfile
        if plugin.jsfile is not None:
            try:
                r = self.jsfiles[plugin.jsfile]
            except KeyError:
                r = self.jsfiles[plugin.jsfile] = JsFile(plugin.jsfile)
            r.content.append((plugin.plugintype, plugin.name))

    def isRegistered(self, plugintype, name):
        'Tells if name is a registered plugin for the plugintype'
        try:
            self.getRegistered(plugintype, name)
        except AzaxPluginError:
            return False
        else:
            return True
    
    def getRegistered(self, plugintype, name):
        'Get the registered plugin and raise error otherwise'
        try:
            plugins_for_class = self.plugins[plugintype]
        except KeyError:
            plugins_for_class = self.plugins[plugintype] = {}
        try:
            plugin = plugins_for_class[name]
        except KeyError:
            raise AzaxPluginError, '"%s" is not a registered plugin for plugin type "%s"' % (name, plugintype)
        return plugin
            
    def checkRegistered(self, plugintype, name):
        'Check if name is a registered plugin and raise error otherwise'
        self.getRegistered(plugintype, name)

    def getAddonFiles(self):
        try:
            files = self._addon_files
        except AttributeError:
            # Lazy setup of addon files
            self._addon_files = files = [] 
            files.extend(self.jsfiles.keys())
        return files
