# -*- coding: ISO-8859-15 -*-
# Copyright (c) 2005-2006
# Authors:
#   Godefroid Chapelle <gotcha@bubblenet.be>
#   Tarek Ziad <tz@nuxeo.com>
#   Balzs Re <ree@greenfinity.hu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
from Testing.ZopeTestCase import ZopeTestCase
from Products.azax import AzaxBaseView
from Products.azax import config 
from ZPublisher.HTTPRequest import HTTPRequest
from cStringIO import StringIO
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from textwrap import dedent

from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.interface import Interface, implements

try:
    # Zope > 2.8
    from zope.app.testing import placelesssetup
except ImportError:
    # Zope == 2.8
    from zope.app.tests import placelesssetup

# Debug request
# This has a modified render.

class IDebugRequest(IBrowserRequest):
    'The debug request'

class DebugTestRequest(TestRequest):
    'Debug test request'
    implements(IDebugRequest)
    def __init__(self):
        self.RESPONSE = DebugTestResponse()
        self.form = {}

class DebugTestResponse:
    def __init__(self, status=200):
        self.status = status
    def getStatus(self):
        return self.status
    def setStatus(self, status):
        self.status = status

# Fake content

class FakeContent(SimpleItem, object):
    pass
    
#InitializeClass(FakeContent)

class AzaxViewTestCase(ZopeTestCase):

    def afterSetUp(self):
        placelesssetup.setUp()
        # fake content
        fakecontent = FakeContent()
        self.folder._setObject('ob', fakecontent)
        fakecontent = fakecontent.__of__(self.folder)
        # Set up a fake view (with no content)
        fakerequest = TestRequest()
        debugfakerequest = DebugTestRequest()
        self.view = AzaxBaseView(fakecontent, fakerequest)
        self.debug_view = AzaxBaseView(fakecontent, debugfakerequest)
        # Allow traversing
        try:
            import Products.Five
        except ImportError:
            # probably zope 3
            pass
            # XXX TODO fix this?? --- never run yet on Z3
            from zope.configuration.xmlconfig import XMLConfig
            import azax
            XMLConfig('tests/configure-unittes.zcml', azax)()
        else:
            from Products.Five.zcml import load_string, load_config
            load_config('meta.zcml', package=Products.Five)
            try:
                # XXX Zope 2.8: must skip this or we explode.
                #raise IOError
                # Zope 2.10 / Five 1.3.6 has traversing.zcml
                load_config('traversing.zcml', package=Products.Five)
            except IOError:
                # on Zope 2.9 we use the old method
                # (this fails on 2.10 since zope.app.traversing is moved)
                load_string(dedent('''\
                        <configure xmlns="http://namespaces.zope.org/zope"
                               xmlns:five="http://namespaces.zope.org/five">
                        <include package="zope.app.traversing" />
                        <adapter
                              for="*"
                              factory="Products.Five.traversable.FiveTraversable"
                              provides="zope.app.traversing.interfaces.ITraversable"
                              />
                        <adapter
                              for="*"
                              factory="zope.app.traversing.adapters.Traverser"
                              provides="zope.app.traversing.interfaces.ITraverser"
                              />
                        </configure>'''))
            # Load the permissions config, if it's there
            try:
                load_config('permissions.zcml', package=Products.Five)
            except IOError:
                # Zope 2.10 / Five 1.3.6 does not have it
                pass
            load_config('meta.zcml', package=Products.azax)
            load_config('configure.zcml', package=Products.azax)
            load_config('configure-unittest.zcml', package=Products.azax.tests)
             
    def beforeTearDown(self):
        placelesssetup.tearDown()
        try:
            import Products.Five
        except ImportError:
            # probably zope 3
            pass
        else:
            import Products.Five.zcml
            Products.Five.zcml._context = None
